@@ -1,8 +1,8 @@
 /*************************************************************************
 ** PSPreviewFilter.cpp                                                  **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,19 +18,18 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <vector>
-#include "InputBuffer.h"
-#include "InputReader.h"
-#include "PSInterpreter.h"
-#include "PSPreviewFilter.h"
-#include "SpecialActions.h"
+#include "InputBuffer.hpp"
+#include "InputReader.hpp"
+#include "PSInterpreter.hpp"
+#include "PSPreviewFilter.hpp"
+#include "SpecialActions.hpp"
 
 
 using namespace std;
 
 PSPreviewFilter::PSPreviewFilter (PSInterpreter &psi)
-	: PSFilter(psi), _active(false), _tightpage(false), _dvi2pt(1.0/65536.0)
+	: PSFilter(psi), _active(false), _tightpage(false), _dvi2bp(1.0/65536.0)
 {
 }
 
@@ -64,6 +63,9 @@ void PSPreviewFilter::execute (const char *code, size_t len) {
 	if (!_tightpage)
 		psInterpreter().execute(code, len);
 	else {
+		// Read bounding box information pushed on the operand stack by the preview package.
+		// It consists of 7 values in DVI units:
+		// llx, lly, urx, ury, height, depth, width
 		CharInputBuffer ib(code, len);
 		BufferInputReader ir(ib);
 		ir.skipSpace();
@@ -81,38 +83,38 @@ void PSPreviewFilter::execute (const char *code, size_t len) {
 bool PSPreviewFilter::getBoundingBox (BoundingBox &bbox) const {
 	if (_boxExtents.size() < 7)
 		return false;
-	double left = -_boxExtents[0]*_dvi2pt;
+	double left = -_boxExtents[0]*_dvi2bp;
 	bbox = BoundingBox(-left, -height(), width()-left, depth());
 	return true;
 }
 
 
-/** Gets the 4 border values set by the preview package.
+/** Gets the 4 border position values set by the preview package (in bp units).
  *  @return true if the border data is available */
 bool PSPreviewFilter::getBorders (double &left, double &right, double &top, double &bottom) const {
 	if (_boxExtents.size() < 4)
 		return false;
-	left   = -_boxExtents[0]*_dvi2pt;
-	top    = -_boxExtents[1]*_dvi2pt;
-	right  = _boxExtents[2]*_dvi2pt;
-	bottom = _boxExtents[3]*_dvi2pt;
+	left   = -_boxExtents[0]*_dvi2bp;
+	top    = -_boxExtents[1]*_dvi2bp;
+	right  = _boxExtents[2]*_dvi2bp;
+	bottom = _boxExtents[3]*_dvi2bp;
 	return true;
 }
 
 
-/** Returns the box height in TeX points, or -1 if no data was found or read yet. */
+/** Returns the box height in PS points, or -1 if no data was found or read yet. */
 double PSPreviewFilter::height () const {
-	return _boxExtents.size() > 4 ? (_boxExtents[4]-_boxExtents[1])*_dvi2pt : -1;
+	return _boxExtents.size() > 4 ? (_boxExtents[4]-_boxExtents[1])*_dvi2bp : -1;
 }
 
 
-/** Returns the box depth in TeX points, or -1 if no data was found or read yet. */
+/** Returns the box depth in PS points, or -1 if no data was found or read yet. */
 double PSPreviewFilter::depth () const {
-	return _boxExtents.size() > 5 ? (_boxExtents[5]+_boxExtents[3])*_dvi2pt : -1;
+	return _boxExtents.size() > 5 ? (_boxExtents[5]+_boxExtents[3])*_dvi2bp : -1;
 }
 
 
-/** Returns the box width in TeX points, or -1 if no data was found or read yet. */
+/** Returns the box width in PS points, or -1 if no data was found or read yet. */
 double PSPreviewFilter::width () const {
-	return _boxExtents.size() > 6 ? (_boxExtents[6]+_boxExtents[2]-_boxExtents[0])*_dvi2pt : -1;
+	return _boxExtents.size() > 6 ? (_boxExtents[6]+_boxExtents[2]-_boxExtents[0])*_dvi2bp : -1;
 }
