@@ -1,8 +1,8 @@
 /*************************************************************************
 ** PsSpecialHandler.cpp                                                 **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,27 +18,21 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <cmath>
 #include <fstream>
-#include <iostream>
 #include <memory>
 #include <sstream>
-#include "EPSFile.h"
-#include "FileFinder.h"
-#include "Ghostscript.h"
-#include "Message.h"
-#include "PathClipper.h"
-#include "PSPattern.h"
-#include "PSPreviewFilter.h"
-#include "PsSpecialHandler.h"
-#include "SpecialActions.h"
-#include "SVGTree.h"
-#include "TensorProductPatch.h"
-#include "VectorIterator.h"
-#include "XMLNode.h"
-#include "XMLString.h"
-#include "TriangularPatch.h"
+#include "EPSFile.hpp"
+#include "FileFinder.hpp"
+#include "Message.hpp"
+#include "PathClipper.hpp"
+#include "PSPattern.hpp"
+#include "PSPreviewFilter.hpp"
+#include "PsSpecialHandler.hpp"
+#include "SpecialActions.hpp"
+#include "SVGTree.hpp"
+#include "TensorProductPatch.hpp"
+#include "TriangularPatch.hpp"
 
 
 using namespace std;
@@ -58,15 +52,13 @@ int PsSpecialHandler::SHADING_SEGMENT_SIZE = 20;
 double PsSpecialHandler::SHADING_SIMPLIFY_DELTA = 0.01;
 
 
-PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _previewFilter(_psi), _psSection(PS_NONE), _xmlnode(0)
+PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(), _previewFilter(_psi), _psSection(PS_NONE), _xmlnode(), _savenode()
 {
 }
 
 
 PsSpecialHandler::~PsSpecialHandler () {
-	_psi.setActions(0);     // ensure no further PS actions are performed
-	for (map<int, PSPattern*>::iterator it=_patterns.begin(); it != _patterns.end(); ++it)
-		delete it->second;
+	_psi.setActions(nullptr);     // ensure no further PS actions are performed
 }
 
 
@@ -75,29 +67,37 @@ PsSpecialHandler::~PsSpecialHandler () {
  *  PS specials. */
 void PsSpecialHandler::initialize () {
 	if (_psSection == PS_NONE) {
-		// initial values of graphics state
-		_linewidth = 1;
-		_linecap = _linejoin = 0;
-		_miterlimit = 4;
-		_xmlnode = _savenode = 0;
-		_opacityalpha = 1;  // fully opaque
-		_sx = _sy = _cos = 1.0;
-		_pattern = 0;
-
+		initgraphics();
 		// execute dvips prologue/header files
-		const char *headers[] = {"tex.pro", "texps.pro", "special.pro", /*"color.pro",*/ 0};
-		for (const char **p=headers; *p; ++p)
-			processHeaderFile(*p);
+		for (const char *fname : {"tex.pro", "texps.pro", "special.pro"})
+			processHeaderFile(fname);
 		// disable bop/eop operators to prevent side-effects by
-		// unexpected bobs/eops present in PS specials
+		// unexpected bops/eops present in PS specials
 		_psi.execute("\nTeXDict begin /bop{pop pop}def /eop{}def end ");
 		_psSection = PS_HEADERS;  // allow to process header specials now
 	}
 }
 
 
+/** Set initial values of PS graphics state (see PS language reference (Red Book)), p. 612). */
+void PsSpecialHandler::initgraphics () {
+	_linewidth = 1;
+	_linecap = _linejoin = 0;  // butt end caps and miter joins
+	_miterlimit = 4;
+	_xmlnode = _savenode = nullptr;
+	_opacityalpha = 1;  // fully opaque
+	_sx = _sy = _cos = 1.0;
+	_pattern = nullptr;
+	_currentcolor = Color::BLACK;
+	_dashoffset = 0;
+	_dashpattern.clear();
+	_path.clear();
+	_clipStack.clear();
+}
+
+
 void PsSpecialHandler::processHeaderFile (const char *name) {
-	if (const char *path = FileFinder::lookup(name, false)) {
+	if (const char *path = FileFinder::instance().lookup(name, false)) {
 		ifstream ifs(path);
 		_psi.execute(string("%%BeginProcSet: ")+name+" 0 0\n", false);
 		_psi.execute(ifs, false);
@@ -170,12 +170,12 @@ void PsSpecialHandler::executeAndSync (istream &is, bool updatePos) {
 }
 
 
-void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions *actions) {
+void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions &actions) {
 	initialize();
 	if (_psSection != PS_HEADERS)
 		return;
 
-	_actions = actions;
+	_actions = &actions;
 	if (*prefix == '!') {
 		_headerCode += "\n";
 		_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
@@ -189,17 +189,17 @@ void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActio
 }
 
 
-bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
+bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions &actions) {
 	// process PS headers only once (in prescan)
 	if (*prefix == '!' || strcmp(prefix, "header=") == 0)
 		return true;
 
-	_actions = actions;
+	_actions = &actions;
 	initialize();
 	if (_psSection != PS_BODY)
 		enterBodySection();
 
-	if (*prefix == '"') {
+	if (*prefix == '"' || strcmp(prefix, "pst:") == 0) {
 		// read and execute literal PostScript code (isolated by a wrapping save/restore pair)
 		moveToDVIPos();
 		_psi.execute("\n@beginspecial @setspecial ");
@@ -210,7 +210,7 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		if (_actions) {
 			StreamInputReader in(is);
 			string fname = in.getQuotedString(in.peek() == '"' ? '"' : 0);
-			map<string,string> attr;
+			unordered_map<string,string> attr;
 			in.parseAttributes(attr);
 			psfile(fname, attr);
 		}
@@ -242,7 +242,7 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 			executeAndSync(is, true);
 		}
 	}
-	else { // ps: ...
+	else { // ps: ... or PST: ...
 		if (_actions)
 			_actions->finishLine();
 		moveToDVIPos();
@@ -269,86 +269,96 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 /** Handles psfile special.
  *  @param[in] fname EPS file to be included
  *  @param[in] attr attributes given with \\special psfile */
-void PsSpecialHandler::psfile (const string &fname, const map<string,string> &attr) {
-	EPSFile epsfile(fname);
-	istream &is = epsfile.istream();
-	if (!is)
+void PsSpecialHandler::psfile (const string &fname, const unordered_map<string,string> &attr) {
+	const char *filepath = FileFinder::instance().lookup(fname, false);
+	if (!filepath) {
 		Message::wstream(true) << "file '" << fname << "' not found in special 'psfile'\n";
-	else {
-		map<string,string>::const_iterator it;
-
-		// bounding box of EPS figure
-		double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second) : 0;
-		double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second) : 0;
-		double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second) : 0;
-		double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second) : 0;
-
-		// desired width/height of resulting figure
-		double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0 : -1;
-		double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0 : -1;
-		if (rwi == 0 || rhi == 0 || urx-llx == 0 || ury-lly == 0)
-			return;
-
-		// user transformations (default values chosen according to dvips manual)
-		double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second) : 0;
-		double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second) : 0;
-//		double hsize   = (it = attr.find("hsize")) != attr.end() ? str2double(it->second) : 612;
-//		double vsize   = (it = attr.find("vsize")) != attr.end() ? str2double(it->second) : 792;
-		double hscale  = (it = attr.find("hscale")) != attr.end() ? str2double(it->second) : 100;
-		double vscale  = (it = attr.find("vscale")) != attr.end() ? str2double(it->second) : 100;
-		double angle   = (it = attr.find("angle")) != attr.end() ? str2double(it->second) : 0;
-
-		Matrix m(1);
-		m.rotate(angle).scale(hscale/100, vscale/100).translate(hoffset, voffset);
-		BoundingBox bbox(llx, lly, urx, ury);
-		bbox.transform(m);
-
-		double sx = rwi/bbox.width();
-		double sy = rhi/bbox.height();
-		if (sx < 0)	sx = sy;
-		if (sy < 0)	sy = sx;
-		if (sx < 0) sx = sy = 1.0;
-
-		// save current DVI position (in pt units)
-		const double x = _actions->getX();
-		const double y = _actions->getY();
-
-		// all following drawings are relative to (0,0)
-		_actions->setX(0);
-		_actions->setY(0);
-		moveToDVIPos();
-
-		_xmlnode = new XMLElementNode("g");
-		_psi.execute("\n@beginspecial @setspecial "); // enter \special environment
-		_psi.limit(epsfile.pslength()); // limit the number of bytes to be processed
-		_psi.execute(is);               // process EPS file
-		_psi.limit(0);                  // disable limitation
-		_psi.execute("\n@endspecial "); // leave special environment
-		if (!_xmlnode->empty()) {       // has anything been drawn?
-			Matrix m(1);
-			m.rotate(angle).scale(hscale/100, vscale/100).translate(hoffset, voffset);
-			m.translate(-llx, lly);
-			m.scale(sx, sy);      // resize image to width "rwi" and height "rhi"
-			m.translate(x, y);    // move image to current DVI position
-			_xmlnode->addAttribute("transform", m.getSVG());
-			_actions->appendToPage(_xmlnode);
-		}
-		else
-			delete _xmlnode;
-		_xmlnode = 0;
-
-		// restore DVI position
-		_actions->setX(x);
-		_actions->setY(y);
-		moveToDVIPos();
-
-		// update bounding box
-		m.scale(sx, -sy);
-		m.translate(x, y);
-		bbox = BoundingBox(0, 0, fabs(urx-llx), fabs(ury-lly));
-		bbox.transform(m);
-		_actions->embed(bbox);
+		return;
 	}
+	unordered_map<string,string>::const_iterator it;
+
+	// bounding box of EPS figure (lower left and upper right corner)
+	double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second) : 0;
+	double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second) : 0;
+	double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second) : 0;
+	double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second) : 0;
+
+	// desired width/height of resulting figure
+	double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0 : -1;
+	double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0 : -1;
+	if (rwi == 0 || rhi == 0 || urx-llx == 0 || ury-lly == 0)
+		return;
+
+	// user transformations (default values chosen according to dvips manual)
+	double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second) : 0;
+	double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second) : 0;
+//	double hsize   = (it = attr.find("hsize")) != attr.end() ? str2double(it->second) : 612;
+//	double vsize   = (it = attr.find("vsize")) != attr.end() ? str2double(it->second) : 792;
+	double hscale  = (it = attr.find("hscale")) != attr.end() ? str2double(it->second) : 100;
+	double vscale  = (it = attr.find("vscale")) != attr.end() ? str2double(it->second) : 100;
+	double angle   = (it = attr.find("angle")) != attr.end() ? str2double(it->second) : 0;
+
+	Matrix m(1);
+	m.rotate(angle).scale(hscale/100, vscale/100).translate(hoffset, voffset);
+	BoundingBox bbox(llx, lly, urx, ury);
+	bbox.transform(m);
+
+	// compute factors to scale the bounding box to width rwi and height rhi
+	double sx = rwi/bbox.width();
+	double sy = rhi/bbox.height();
+	if (sx == 0 || sy == 0)
+		return;
+
+	if (sx < 0) sx = sy;         // rwi attribute not set
+	if (sy < 0) sy = sx;         // rhi attribute not set
+	if (sx < 0) sx = sy = 1.0;   // neither rwi nor rhi set
+
+	// save current DVI position
+	const double x = _actions->getX();
+	const double y = _actions->getY();
+
+	// all following drawings are relative to (0,0)
+	_actions->setX(0);
+	_actions->setY(0);
+	moveToDVIPos();
+
+	// transform current DVI position and bounding box location
+	// according to current transformation matrix
+	DPair llTrans = _actions->getMatrix()*DPair(llx, -lly);
+	DPair urTrans = _actions->getMatrix()*DPair(urx, -ury);
+	DPair dviposTrans = _actions->getMatrix()*DPair(x, y);
+
+	auto groupNode = util::make_unique<XMLElementNode>("g");  // append following elements to this group
+	_xmlnode = groupNode.get();
+	_psi.execute("\n@beginspecial @setspecial "); // enter \special environment
+	EPSFile epsfile(filepath);
+	_psi.limit(epsfile.pslength());  // limit the number of bytes going to be processed
+	_psi.execute(epsfile.istream()); // process EPS file
+	_psi.limit(0);                   // disable limitation
+	_psi.execute("\n@endspecial ");  // leave special environment
+	if (!groupNode->empty()) {       // has anything been drawn?
+		Matrix matrix(1);
+		matrix.rotate(angle).scale(hscale/100, vscale/100).translate(hoffset, voffset);
+		matrix.translate(-llTrans);
+		matrix.scale(sx, sy);          // resize image to width "rwi" and height "rhi"
+		matrix.translate(dviposTrans); // move image to current DVI position
+		if (!matrix.isIdentity())
+			groupNode->addAttribute("transform", matrix.getSVG());
+		_actions->appendToPage(std::move(groupNode));
+	}
+	_xmlnode = nullptr;   // append following elements to page group again
+
+	// restore DVI position
+	_actions->setX(x);
+	_actions->setY(y);
+	moveToDVIPos();
+
+	// update bounding box
+	m.scale(sx, -sy);
+	m.translate(dviposTrans);
+	bbox = BoundingBox(DPair(0, 0), abs(urTrans-llTrans));
+	bbox.transform(m);
+	_actions->embed(bbox);
 }
 
 
@@ -380,52 +390,97 @@ static bool transform_box_extents (const Matrix &matrix, double &w, double &h, d
 }
 
 
-void PsSpecialHandler::dviEndPage (unsigned) {
+void PsSpecialHandler::dviEndPage (unsigned, SpecialActions &actions) {
 	BoundingBox bbox;
-	if (_previewFilter.getBoundingBox(bbox)) {
-		double w = _previewFilter.width();
-		double h = _previewFilter.height();
-		double d = _previewFilter.depth();
-		bool horiz_baseline = true;
-		if (_actions) {
-			_actions->bbox() = bbox;
+	if (_previewFilter.getBoundingBox(bbox)) {  // is there any data written by preview package?
+		double w = max(0.0, _previewFilter.width());
+		double h = max(0.0, _previewFilter.height());
+		double d = max(0.0, _previewFilter.depth());
+		if (actions.getBBoxFormatString() == "preview" || actions.getBBoxFormatString() == "min") {
+			if (actions.getBBoxFormatString() == "preview") {
+				actions.bbox() = bbox;
+				Message::mstream() << "\napplying bounding box set by";
+			}
+			else {
+				// compute height, depth and width depending on the
+				// tight bounding box derived from the objects on the page
+				double y0 = bbox.maxY()-h;      // y coordinate of the baseline
+				h = actions.bbox().maxY()-y0;
+				if (h < 0) {
+					h = 0;
+					d = actions.bbox().height();
+				}
+				else {
+					d = y0-actions.bbox().minY();
+					if (d < 0) {
+						h = actions.bbox().height();
+						d = 0;
+					}
+				}
+				w = actions.bbox().width();
+				Message::mstream() << "\ncomputing extents based on data set by";
+			}
+			Message::mstream() << " preview package (version " << _previewFilter.version() << ")\n";
+
 			// apply page transformations to box extents
 			Matrix pagetrans;
-			_actions->getPageTransform(pagetrans);
-			horiz_baseline = transform_box_extents(pagetrans, w, h, d);
-			_actions->bbox().lock();
+			actions.getPageTransform(pagetrans);
+			bool isBaselineHorizontal = transform_box_extents(pagetrans, w, h, d);
+			actions.bbox().lock();
+
+			if (!isBaselineHorizontal)
+				Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
+			else {
+				const double bp2pt = 72.27/72.0;
+				Message::mstream() <<
+					"width=" << XMLString(w*bp2pt) << "pt, "
+					"height=" << XMLString(h*bp2pt) << "pt, "
+					"depth=" << XMLString(d*bp2pt) << "pt\n";
+			}
 		}
-		Message::mstream() << "\napplying bounding box set by preview package (version " << _previewFilter.version() << ")\n";
-		if (horiz_baseline) {
-			const double bp2pt = 72.27/72.0;
-			Message::mstream() <<
-				"width=" << XMLString(w*bp2pt) << "pt, "
-				"height=" << XMLString(h*bp2pt) << "pt, "
-				"depth=" << XMLString(d*bp2pt) << "pt\n";
+#if 0
+		XMLElementNode *rect = new XMLElementNode("rect");
+		rect->addAttribute("x", actions.bbox().minX());
+		rect->addAttribute("y", actions.bbox().minY());
+		rect->addAttribute("width", w);
+		rect->addAttribute("height", h+d);
+		rect->addAttribute("fill", "none");
+		rect->addAttribute("stroke", "red");
+		rect->addAttribute("stroke-width", "0.5");
+		actions.appendToPage(rect);
+		if (d > 0) {
+			XMLElementNode *line = new XMLElementNode("line");
+			line->addAttribute("x1", actions.bbox().minX());
+			line->addAttribute("y1", actions.bbox().minY()+h);
+			line->addAttribute("x2", actions.bbox().maxX());
+			line->addAttribute("y2", actions.bbox().minY()+h);
+			line->addAttribute("stroke", "blue");
+			line->addAttribute("stroke-width", "0.5");
+			actions.appendToPage(line);
 		}
-		else
-			Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
+#endif
 	}
 	// close dictionary TeXDict and execute end-hook if defined
 	if (_psSection == PS_BODY) {
-		_psi.execute("\nend userdict/end-hook known{end-hook}if ");
+		_psi.execute("\nend userdict/end-hook known{end-hook}if initgraphics ");
+		initgraphics();  // reset graphics state to default values
 		_psSection = PS_HEADERS;
 	}
 }
 
 ///////////////////////////////////////////////////////
 
-void PsSpecialHandler::gsave (vector<double> &p) {
+void PsSpecialHandler::gsave (vector<double>&) {
 	_clipStack.dup();
 }
 
 
-void PsSpecialHandler::grestore (vector<double> &p) {
+void PsSpecialHandler::grestore (vector<double>&) {
 	_clipStack.pop();
 }
 
 
-void PsSpecialHandler::grestoreall (vector<double> &p) {
+void PsSpecialHandler::grestoreall (vector<double>&) {
 	_clipStack.pop(-1, true);
 }
 
@@ -455,7 +510,7 @@ void PsSpecialHandler::curveto (vector<double> &p) {
 }
 
 
-void PsSpecialHandler::closepath (vector<double> &p) {
+void PsSpecialHandler::closepath (vector<double>&) {
 	_path.closepath();
 }
 
@@ -463,6 +518,7 @@ void PsSpecialHandler::closepath (vector<double> &p) {
 /** Draws the current path recorded by previously executed path commands (moveto, lineto,...).
  *  @param[in] p not used */
 void PsSpecialHandler::stroke (vector<double> &p) {
+	_path.removeRedundantCommands();
 	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
 		return;
 
@@ -474,14 +530,14 @@ void PsSpecialHandler::stroke (vector<double> &p) {
 	}
 	if (_clipStack.clippathLoaded() && _clipStack.top())
 		_path.prepend(*_clipStack.top());
-	XMLElementNode *path=0;
+	unique_ptr<XMLElementNode> path;
 	Pair<double> point;
 	if (_path.isDot(point)) {  // zero-length path?
 		if (_linecap == 1) {    // round line ends?  => draw dot
 			double x = point.x();
 			double y = point.y();
 			double r = _linewidth/2.0;
-			path = new XMLElementNode("circle");
+			path = util::make_unique<XMLElementNode>("circle");
 			path->addAttribute("cx", x);
 			path->addAttribute("cy", y);
 			path->addAttribute("r", r);
@@ -496,7 +552,7 @@ void PsSpecialHandler::stroke (vector<double> &p) {
 
 		ostringstream oss;
 		_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
-		path = new XMLElementNode("path");
+		path = util::make_unique<XMLElementNode>("path");
 		path->addAttribute("d", oss.str());
 		path->addAttribute("stroke", _actions->getColor().svgColorString());
 		path->addAttribute("fill", "none");
@@ -531,9 +587,9 @@ void PsSpecialHandler::stroke (vector<double> &p) {
 		_clipStack.setClippathLoaded(false);
 	}
 	if (_xmlnode)
-		_xmlnode->append(path);
+		_xmlnode->append(std::move(path));
 	else {
-		_actions->appendToPage(path);
+		_actions->appendToPage(std::move(path));
 		_actions->embed(bbox);
 	}
 	_path.clear();
@@ -544,6 +600,7 @@ void PsSpecialHandler::stroke (vector<double> &p) {
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
 void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
+	_path.removeRedundantCommands();
 	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
 		return;
 
@@ -560,7 +617,7 @@ void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
 
 	ostringstream oss;
 	_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
-	XMLElementNode *path = new XMLElementNode("path");
+	unique_ptr<XMLElementNode> path = util::make_unique<XMLElementNode>("path");
 	path->addAttribute("d", oss.str());
 	if (_pattern)
 		path->addAttribute("fill", XMLString("url(#")+_pattern->svgID()+")");
@@ -579,9 +636,9 @@ void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
 	if (_opacityalpha < 1)
 		path->addAttribute("fill-opacity", _opacityalpha);
 	if (_xmlnode)
-		_xmlnode->append(path);
+		_xmlnode->append(std::move(path));
 	else {
-		_actions->appendToPage(path);
+		_actions->appendToPage(std::move(path));
 		_actions->embed(bbox);
 	}
 	_path.clear();
@@ -630,7 +687,7 @@ void PsSpecialHandler::makepattern (vector<double> &p) {
 			// pattern definition completed
 			if (_savenode) {
 				_xmlnode = _savenode;
-				_savenode = 0;
+				_savenode = nullptr;
 			}
 			break;
 		case 1: {  // tiling pattern
@@ -642,14 +699,14 @@ void PsSpecialHandler::makepattern (vector<double> &p) {
 			create_matrix(p, 9, matrix);
 			matrix.rmultiply(_actions->getMatrix());
 
-			PSTilingPattern *pattern=0;
+			unique_ptr<PSTilingPattern> pattern;
 			if (paint_type == 1)
-				pattern = new PSColoredTilingPattern(id, bbox, matrix, xstep, ystep);
+				pattern = util::make_unique<PSColoredTilingPattern>(id, bbox, matrix, xstep, ystep);
 			else
-				pattern = new PSUncoloredTilingPattern(id, bbox, matrix, xstep, ystep);
-			_patterns[id] = pattern;
+				pattern = util::make_unique<PSUncoloredTilingPattern>(id, bbox, matrix, xstep, ystep);
 			_savenode = _xmlnode;
 			_xmlnode = pattern->getContainerNode();  // insert the following SVG elements into this node
+			_patterns[id] = std::move(pattern);
 			break;
 		}
 		case 2: {
@@ -664,21 +721,21 @@ void PsSpecialHandler::makepattern (vector<double> &p) {
  *  1-3: (optional) RGB values for uncolored tiling patterns
  *  further parameters depend on the pattern type */
 void PsSpecialHandler::setpattern (vector<double> &p) {
-	int pattern_id = p[0];
+	int patternID = static_cast<int>(p[0]);
 	Color color;
 	if (p.size() == 4)
 		color.setRGB(p[1], p[2], p[3]);
-	map<int,PSPattern*>::iterator it = _patterns.find(pattern_id);
+	auto it = _patterns.find(patternID);
 	if (it == _patterns.end())
-		_pattern = 0;
+		_pattern = nullptr;
 	else {
-		if (PSUncoloredTilingPattern *pattern = dynamic_cast<PSUncoloredTilingPattern*>(it->second))
+		if (auto *pattern = dynamic_cast<PSUncoloredTilingPattern*>(it->second.get()))
 			pattern->setColor(color);
-		it->second->apply(_actions);
-		if (PSTilingPattern *pattern = dynamic_cast<PSTilingPattern*>(it->second))
+		it->second->apply(*_actions);
+		if (auto *pattern = dynamic_cast<PSTilingPattern*>(it->second.get()))
 			_pattern = pattern;
 		else
-			_pattern = 0;
+			_pattern = nullptr;
 	}
 }
 
@@ -705,7 +762,7 @@ void PsSpecialHandler::clippath (std::vector<double>&) {
  *  path (see PS language reference, 3rd edition, pp. 193, 542)
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
-void PsSpecialHandler::clip (vector<double> &, bool evenodd) {
+void PsSpecialHandler::clip (vector<double>&, bool evenodd) {
 	clip(_path, evenodd);
 }
 
@@ -716,11 +773,11 @@ void PsSpecialHandler::clip (vector<double> &, bool evenodd) {
  *  @param[in] path path used to restrict the clipping region
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
 void PsSpecialHandler::clip (Path &path, bool evenodd) {
-		// when this method is called, _path contains the clipping path
+	// when this method is called, _path contains the clipping path
 	if (path.empty() || !_actions)
 		return;
 
-	Path::WindingRule windingRule = evenodd ? Path::WR_EVEN_ODD : Path::WR_NON_ZERO;
+	Path::WindingRule windingRule = evenodd ? Path::WindingRule::EVEN_ODD : Path::WindingRule::NON_ZERO;
 	path.setWindingRule(windingRule);
 
 	if (!_actions->getMatrix().isIdentity())
@@ -735,7 +792,7 @@ void PsSpecialHandler::clip (Path &path, bool evenodd) {
 	}
 	else {
 		// compute the intersection of the current clipping path with the current graphics path
-		Path *oldPath = _clipStack.getPath(oldID);
+		const Path *oldPath = _clipStack.top();
 		Path intersectedPath(windingRule);
 		PathClipper clipper;
 		clipper.intersect(*oldPath, path, intersectedPath);
@@ -743,19 +800,19 @@ void PsSpecialHandler::clip (Path &path, bool evenodd) {
 		intersectedPath.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
 	}
 
-	XMLElementNode *pathElem = new XMLElementNode("path");
+	auto pathElem = util::make_unique<XMLElementNode>("path");
 	pathElem->addAttribute("d", oss.str());
 	if (evenodd)
 		pathElem->addAttribute("clip-rule", "evenodd");
 
 	int newID = _clipStack.topID();
-	XMLElementNode *clipElem = new XMLElementNode("clipPath");
+	auto clipElem = util::make_unique<XMLElementNode>("clipPath");
 	clipElem->addAttribute("id", XMLString("clip")+XMLString(newID));
 	if (!COMPUTE_CLIPPATHS_INTERSECTIONS && oldID)
 		clipElem->addAttribute("clip-path", XMLString("url(#clip")+XMLString(oldID)+")");
 
-	clipElem->append(pathElem);
-	_actions->appendToDefs(clipElem);
+	clipElem->append(std::move(pathElem));
+	_actions->appendToDefs(std::move(clipElem));
 }
 
 
@@ -772,11 +829,11 @@ void PsSpecialHandler::shfill (vector<double> &params) {
 
 	// collect common data relevant for all shading types
 	int shadingTypeID = static_cast<int>(params[0]);
-	ColorSpace colorSpace = Color::RGB_SPACE;
+	ColorSpace colorSpace = Color::ColorSpace::RGB;
 	switch (static_cast<int>(params[1])) {
-		case 1: colorSpace = Color::GRAY_SPACE; break;
-		case 3: colorSpace = Color::RGB_SPACE; break;
-		case 4: colorSpace = Color::CMYK_SPACE; break;
+		case 1: colorSpace = Color::ColorSpace::GRAY; break;
+		case 3: colorSpace = Color::ColorSpace::RGB; break;
+		case 4: colorSpace = Color::ColorSpace::CMYK; break;
 	}
 	VectorIterator<double> it = params;
 	it += 2;     // skip shading type and color space
@@ -865,32 +922,34 @@ static void read_patch_data (ShadingPatch &patch, int edgeflag,
 
 class ShadingCallback : public ShadingPatch::Callback {
 	public:
-		ShadingCallback (SpecialActions *actions, XMLElementNode *parent, int clippathID)
-			: _actions(actions), _group(new XMLElementNode("g"))
+		ShadingCallback (SpecialActions &actions, XMLElementNode *parent, int clippathID)
+			: _actions(actions)
 		{
+			auto group = util::make_unique<XMLElementNode>("g");
+			_group = group.get();
 			if (parent)
-				parent->append(_group);
+				parent->append(std::move(group));
 			else
-				actions->appendToPage(_group);
+				actions.appendToPage(std::move(group));
 			if (clippathID > 0)
 				_group->addAttribute("clip-path", XMLString("url(#clip")+XMLString(clippathID)+")");
 		}
 
-		void patchSegment (GraphicPath<double> &path, const Color &color) {
-			if (!_actions->getMatrix().isIdentity())
-				path.transform(_actions->getMatrix());
+		void patchSegment (GraphicsPath<double> &path, const Color &color) override {
+			if (!_actions.getMatrix().isIdentity())
+				path.transform(_actions.getMatrix());
 
 			// draw a single patch segment
 			ostringstream oss;
 			path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
-			XMLElementNode *pathElem = new XMLElementNode("path");
+			auto pathElem = util::make_unique<XMLElementNode>("path");
 			pathElem->addAttribute("d", oss.str());
 			pathElem->addAttribute("fill", color.svgColorString());
-			_group->append(pathElem);
+			_group->append(std::move(pathElem));
 		}
 
 	private:
-		SpecialActions *_actions;
+		SpecialActions &_actions;
 		XMLElementNode *_group;
 };
 
@@ -898,22 +957,20 @@ class ShadingCallback : public ShadingPatch::Callback {
 /** Handle all patch meshes whose patches and their connections can be processed sequentially.
  *  This comprises free-form triangular, Coons, and tensor-product patch meshes. */
 void PsSpecialHandler::processSequentialPatchMesh (int shadingTypeID, ColorSpace colorSpace, VectorIterator<double> &it) {
-	auto_ptr<ShadingPatch> previousPatch;
+	unique_ptr<ShadingPatch> previousPatch;
 	while (it.valid()) {
 		int edgeflag = static_cast<int>(*it++);
 		vector<DPair> points;
 		vector<Color> colors;
-		auto_ptr<ShadingPatch> patch;
-
-		patch = auto_ptr<ShadingPatch>(ShadingPatch::create(shadingTypeID, colorSpace));
+		unique_ptr<ShadingPatch> patch = ShadingPatch::create(shadingTypeID, colorSpace);
 		read_patch_data(*patch, edgeflag, it, points, colors);
 		patch->setPoints(points, edgeflag, previousPatch.get());
 		patch->setColors(colors, edgeflag, previousPatch.get());
-		ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+		ShadingCallback callback(*_actions, _xmlnode, _clipStack.topID());
 #if 0
 		if (bgcolorGiven) {
 			// fill whole patch area with given background color
-			GraphicPath<double> outline;
+			GraphicsPath<double> outline;
 			patch->getBoundaryPath(outline);
 			callback.patchSegment(outline, bgcolor);
 		}
@@ -926,7 +983,7 @@ void PsSpecialHandler::processSequentialPatchMesh (int shadingTypeID, ColorSpace
 			bbox.transform(_actions->getMatrix());
 			_actions->embed(bbox);
 		}
-		previousPatch = patch;
+		previousPatch = std::move(patch);
 	}
 }
 
@@ -955,7 +1012,7 @@ void PsSpecialHandler::processLatticeTriangularPatchMesh (ColorSpace colorSpace,
 		vertex.color.set(colorSpace, it);
 	}
 	LatticeTriangularPatch patch(colorSpace);
-	ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+	ShadingCallback callback(*_actions, _xmlnode, _clipStack.topID());
 	while (it.valid()) {
 		// read next row
 		for (int i=0; i < verticesPerRow; i++) {
@@ -1087,22 +1144,20 @@ void PsSpecialHandler::executed () {
 
 void PsSpecialHandler::ClippingStack::pushEmptyPath () {
 	if (!_stack.empty())
-		_stack.push(Entry(0, -1));
+		_stack.emplace(Entry());
 }
 
 
 void PsSpecialHandler::ClippingStack::push (const Path &path, int saveID) {
 	if (path.empty())
-		_stack.push(Entry(0, saveID));
-	else {
-		_paths.push_back(path);
-		_stack.push(Entry(_paths.size(), saveID));
-	}
+		_stack.emplace(Entry(saveID));
+	else
+		_stack.emplace(Entry(path, ++_maxID, saveID));
 }
 
 
 /** Pops a single or several elements from the clipping stack.
- *  The method distingushes between the following cases:
+ *  The method distinguishes between the following cases:
  *  1) saveID < 0 and grestoreall == false:
  *     pop top element if it was pushed by gsave (its saveID is < 0 as well)
  *  2) saveID < 0 and grestoreall == true
@@ -1111,36 +1166,30 @@ void PsSpecialHandler::ClippingStack::push (const Path &path, int saveID) {
  *  3) saveID >= 0:
  *     pop all elements until the saveID of the top element equals parameter saveID */
 void PsSpecialHandler::ClippingStack::pop (int saveID, bool grestoreall) {
-	if (!_stack.empty()) {
-		if (saveID < 0) {                // grestore?
-			if (_stack.top().saveID < 0)  // pushed by 'gsave'?
-				_stack.pop();
-			// pop all further elements pushed by 'gsave' if grestoreall == true
-			while (grestoreall && !_stack.empty() && _stack.top().saveID < 0)
-				_stack.pop();
-		}
-		else {
-			// pop elements pushed by 'gsave'
-			while (!_stack.empty() && _stack.top().saveID != saveID)
-				_stack.pop();
-			// pop element pushed by 'save'
-			if (!_stack.empty())
-				_stack.pop();
-		}
+	if (_stack.empty())
+		return;
+
+	if (saveID < 0) {                // grestore?
+		if (_stack.top().saveID < 0)  // pushed by 'gsave'?
+			_stack.pop();
+		// pop all further elements pushed by 'gsave' if grestoreall == true
+		while (grestoreall && !_stack.empty() && _stack.top().saveID < 0)
+			_stack.pop();
+	}
+	else {
+		// pop elements pushed by 'gsave'
+		while (!_stack.empty() && _stack.top().saveID != saveID)
+			_stack.pop();
+		// pop element pushed by 'save'
+		if (!_stack.empty())
+			_stack.pop();
 	}
 }
 
 
 /** Returns a pointer to the path on top of the stack, or 0 if the stack is empty. */
 const PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::top () const {
-	return (!_stack.empty() && _stack.top().pathID)
-		? &_paths[_stack.top().pathID-1]
-		: 0;
-}
-
-
-PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::getPath (size_t id) {
-	return (id > 0 && id <= _paths.size()) ? &_paths[id-1] : 0;
+	return _stack.empty() ? nullptr : _stack.top().path.get();
 }
 
 
@@ -1159,7 +1208,6 @@ void PsSpecialHandler::ClippingStack::setClippathLoaded (bool loaded) {
 
 /** Pops all elements from the stack. */
 void PsSpecialHandler::ClippingStack::clear() {
-	_paths.clear();
 	while (!_stack.empty())
 		_stack.pop();
 }
@@ -1171,20 +1219,30 @@ void PsSpecialHandler::ClippingStack::replace (const Path &path) {
 	if (_stack.empty())
 		push(path, -1);
 	else {
-		_paths.push_back(path);
-		_stack.top().pathID = _paths.size();
+		_stack.top().path = make_shared<Path>(path);
+		_stack.top().pathID = ++_maxID;
 	}
 }
 
 
 /** Duplicates the top element, i.e. the top element is pushed again. */
 void PsSpecialHandler::ClippingStack::dup (int saveID) {
-	_stack.push(_stack.empty() ? Entry(0, -1) : _stack.top());
+	_stack.emplace(_stack.empty() ? Entry() : _stack.top());
 	_stack.top().saveID = saveID;
 }
 
 
-const char** PsSpecialHandler::prefixes () const {
-	static const char *pfx[] = {"header=", "psfile=", "PSfile=", "ps:", "ps::", "!", "\"", 0};
+const vector<const char*> PsSpecialHandler::prefixes () const {
+	const vector<const char*> pfx {
+		"header=",    // read and execute PS header file prior to the following PS statements
+		"psfile=",    // read and execute PS file
+		"PSfile=",    // dito
+		"ps:",        // execute literal PS code wrapped by @beginspecial and @endspecial
+		"ps::",       // execute literal PS code without additional adaption of the drawing position
+		"!",          // execute literal PS header code following this prefix
+		"\"",         // execute literal PS code following this prefix
+		"pst:",       // dito
+		"PST:",       // same as "ps:"
+	};
 	return pfx;
 }
