@@ -1,8 +1,8 @@
 /*************************************************************************
 ** Subfont.cpp                                                          **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,21 +18,23 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
 #include <limits>
-#include "FileFinder.h"
-#include "Subfont.h"
-#include "Message.h"
+#include <memory>
+#include "FileFinder.hpp"
+#include "Message.hpp"
+#include "Subfont.hpp"
+#include "utility.hpp"
+
 
 using namespace std;
 
 // helper functions
 
 static int skip_mapping_data (istream &is);
-static bool scan_line (const char *line, int lineno, UInt16 *mapping, const string &fname, int &pos);
+static bool scan_line (const char *line, int lineno, vector<uint16_t> &mapping, const string &fname, long &pos);
 
 
 /** Constructs a new SubfontDefinition object.
@@ -51,9 +53,9 @@ SubfontDefinition::SubfontDefinition (const string &name, const char *fpath) : _
 			while (is && !isspace(is.peek()))
 				id += is.get();
 			if (!id.empty()) {
-				pair<Iterator, bool> state = _subfonts.insert(pair<string,Subfont*>(id, (Subfont*)0));
+				auto state = _subfonts.emplace(pair<string,unique_ptr<Subfont>>(id, unique_ptr<Subfont>()));
 				if (state.second) // id was not present in map already
-					state.first->second = new Subfont(*this, state.first->first);
+					state.first->second = unique_ptr<Subfont>(new Subfont(*this, state.first->first));
 				skip_mapping_data(is);
 			}
 		}
@@ -61,26 +63,19 @@ SubfontDefinition::SubfontDefinition (const string &name, const char *fpath) : _
 }
 
 
-SubfontDefinition::~SubfontDefinition () {
-	for (Iterator it=_subfonts.begin(); it != _subfonts.end(); ++it)
-		delete it->second;
-}
-
-
 /** Looks for a subfont definition of the given name and returns the corresponding object.
  *  All objects are only created once for a given name and stored in an internal cache.
  *  @param[in] name name of subfont definition to lookup
  *  @return pointer to subfont definition object or 0 if it doesn't exist */
 SubfontDefinition* SubfontDefinition::lookup (const std::string &name) {
-   typedef map<string,SubfontDefinition*> SFDMap;
-   static SFDMap sfdMap;
-   SFDMap::iterator it = sfdMap.find(name);
-   if (it != sfdMap.end())
-      return it->second;
-	SubfontDefinition *sfd=0;
-	if (const char *path = FileFinder::lookup(name+".sfd", false)) {
+	static map<string,unique_ptr<SubfontDefinition>> sfdMap;
+	auto it = sfdMap.find(name);
+	if (it != sfdMap.end())
+		return it->second.get();
+	SubfontDefinition *sfd=nullptr;
+	if (const char *path = FileFinder::instance().lookup(name+".sfd", false)) {
 		sfd = new SubfontDefinition(name, path);
-		sfdMap[name] = sfd;
+		sfdMap[name] = unique_ptr<SubfontDefinition>(sfd);
 	}
 	return sfd;
 }
@@ -88,32 +83,28 @@ SubfontDefinition* SubfontDefinition::lookup (const std::string &name) {
 
 /** Returns the full path to the corresponding .sfd file or 0 if it can't be found. */
 const char* SubfontDefinition::path () const {
-   return FileFinder::lookup(filename(), false);
+	return FileFinder::instance().lookup(filename(), false);
 }
 
 
 /** Returns the subfont with the given ID, or 0 if it doesn't exist. */
 Subfont* SubfontDefinition::subfont (const string &id) const {
-   ConstIterator it = _subfonts.find(id);
-   if (it != _subfonts.end())
-      return it->second;
-   return 0;
+	auto it = _subfonts.find(id);
+	if (it != _subfonts.end())
+		return it->second.get();
+	return nullptr;
 }
 
 
 /** Returns all subfonts defined in this SFD. */
 int SubfontDefinition::subfonts (vector<Subfont*> &sfs) const {
-   for (ConstIterator it=_subfonts.begin(); it != _subfonts.end(); ++it)
-      sfs.push_back(it->second);
-   return sfs.size();
+	for (const auto &strsfpair : _subfonts)
+		sfs.push_back(strsfpair.second.get());
+	return int(sfs.size());
 }
 
 //////////////////////////////////////////////////////////////////////
 
-Subfont::~Subfont () {
-   delete [] _mapping;
-}
-
 
 /** Reads the character mappings for a given subfont ID.
  *  Format of subfont definition (sfd) files:
@@ -128,13 +119,12 @@ Subfont::~Subfont () {
  *  to c=10, 11 and 12, respectively.
  *  @return true if the data has been read successfully */
 bool Subfont::read () {
-  	if (_mapping)  // if there's already a mapping assigned, we're finished here
+	if (!_mapping.empty())  // if there's already a mapping assigned, we're finished here
 		return true;
 	if (const char *p = _sfd.path()) {
 		ifstream is(p);
 		if (!is)
 			return false;
-
 		int lineno=1;
 		while (is) {
 			if (is.peek() == '#' || is.peek() == '\n') {
@@ -151,14 +141,13 @@ bool Subfont::read () {
 					lineno += skip_mapping_data(is);
 				else {
 					// build mapping array
-					_mapping = new UInt16[256];
-					memset(_mapping, 0, 256*sizeof(UInt16));
-					int pos=0;
+					_mapping.resize(256, 0);
+					long pos=0;
 					char buf[1024];
 					bool complete=false;
 					while (!complete) {
 						is.getline(buf, 1024);
-						complete = scan_line(buf, lineno, _mapping, _sfd.filename() ,pos);
+						complete = scan_line(buf, lineno, _mapping, _sfd.filename(), pos);
 					}
 					return true;
 				}
@@ -173,8 +162,8 @@ bool Subfont::read () {
  *  (local) character code of the subfont.
  *  @param[in] c local character code relative to the subfont
  *  @return character code of the target font */
-UInt16 Subfont::decode (unsigned char c) {
-	if (!_mapping && !read())
+uint16_t Subfont::decode (unsigned char c) {
+	if (_mapping.empty() && !read())
 		return 0;
 	return _mapping[c];
 }
@@ -184,7 +173,7 @@ UInt16 Subfont::decode (unsigned char c) {
 
 
 /** Skips the mapping data of a subfont entry.
- *  @param[in] stream to read from
+ *  @param[in] is stream to read from
  *  @return number of lines skipped */
 static int skip_mapping_data (istream &is) {
 	char buf[1024];
@@ -207,9 +196,10 @@ static int skip_mapping_data (istream &is) {
  *  @param[in] line the line of text to be scanned
  *  @param[in] lineno line number used in exception messages
  *  @param[in,out] mapping the mapping data
- *  @param[in,out] offest position/index of next mapping value
+ *  @param[in] fname name of the mapfile being scanned
+ *  @param[in,out] offset position/index of next mapping value
  *  @return true if the line is the last one the current mapping sequence, i.e. the line doesn't end with a backslash */
-static bool scan_line (const char *line, int lineno, UInt16 *mapping, const string &fname, int &offset) {
+static bool scan_line (const char *line, int lineno, vector<uint16_t> &mapping, const string &fname, long &offset) {
 	const char *p=line;
 	char *q;
 	for (; *p && isspace(*p); p++);
@@ -221,12 +211,11 @@ static bool scan_line (const char *line, int lineno, UInt16 *mapping, const stri
 		}
 		else {
 			long val1 = strtol(p, &q, 0); // first value of range
-			long val2;                    // last value of range
-			ostringstream oss; // output stream for exception messages
+			long val2 = -1;               // last value of range
 			switch (*q) {
 				case ':':
 					if (val1 < 0 || val1 > 255)
-						throw SubfontException(oss << "offset value " << val1 << " out of range (0-255)", fname, lineno);
+						throw SubfontException("offset value "+to_string(val1)+" out of range (0-255)", fname, lineno);
 					offset = val1;
 					val1 = -1;
 					q++;
@@ -235,26 +224,26 @@ static bool scan_line (const char *line, int lineno, UInt16 *mapping, const stri
 					p = q+1;
 					val2 = strtol(p, &q, 0);
 					if (val1 < 0 || val1 > 0xffffL)
-						throw SubfontException(oss << "table value " << val1 << " out of range", fname, lineno);
+						throw SubfontException("table value "+to_string(val1)+" out of range", fname, lineno);
 					if (val2 < 0 || val2 > 0xffffL)
-						throw SubfontException(oss << "table value " << val2 << " out of range", fname, lineno);
+						throw SubfontException("table value "+to_string(val2)+" out of range", fname, lineno);
 					if (p == q || (!isspace(*q) && *q != '\\' && *q))
-						throw SubfontException(oss << "unexpected character '" << *q << "'", fname, lineno);
+						throw SubfontException("unexpected character '"+to_string(*q)+"'", fname, lineno);
 					break;
 				default:
 					if (p == q || (!isspace(*q) && *q != '\\' && *q))
-						throw SubfontException(oss << "unexpected character '" << *q << "'", fname, lineno);
+						throw SubfontException("unexpected character '"+to_string(*q)+"'", fname, lineno);
 					if (val1 < 0 || val1 > 0xffffL)
 						throw SubfontException("invalid character code", fname, lineno);
 					val2 = val1;
 			}
 			if (val1 >= 0) {
 				if (val1 > val2 || offset+val2-val1 > 255)
-					throw SubfontException(oss << "invalid range in mapping table: " << hex << val1 << '_' << val2, fname, lineno);
+					throw SubfontException("invalid range in mapping table: "+util::tohex(val1)+"_"+util::tohex(val2), fname, lineno);
 				for (long v=val1; v <= val2; v++) {
 					if (mapping[offset])
-						throw SubfontException(oss << "mapping of character " << offset << " already defined", fname, lineno);
-					mapping[offset++] = static_cast<UInt16>(v);
+						throw SubfontException("mapping of character "+to_string(offset)+" already defined", fname, lineno);
+					mapping[offset++] = static_cast<uint16_t>(v);
 				}
 			}
 			for (p=q; *p && isspace(*p); p++);
